/*
 *  GTCommonAPI.cpp
 *  All
 *
 *  Created by nhn on 11. 1. 19..
 *  Copyright 2011 __MyCompanyName__. All rights reserved.
 *
 */
#include "StdAfx.h"
#include "GTCommonAPI.h"
#include <fcntl.h>
#include <ctype.h>
#include <unistd.h>
#include <ctime>
#include "GtLog.h"
#include <errno.h>
#include <sys/stat.h>
#include "JNIHelper.h"
#include "BindUtil.h"
#include "BindJavaFile.h"
#include "BindJavaAndroidContext.h"

static char g_szUUID[128];

#define COMMONAPI_ASSERT(cond)

#define COMMONAPI_LOG(...)	LOGE2("GTCOMMONAPI", __VA_ARGS__)

//static jclass __getJNIFileManager(JNIEnv **env)
//{
//	static jclass clazz;
//	JNIEnv *jniEnv;
//	jniEnv = jniGetJNIEnvOnCurrentThreads();
//
//	if(env)
//		*env = jniEnv;
//
//	if(NULL == jniEnv)
//	{
//		return NULL;
//	}
//
//	if(NULL == clazz)
//	{
//		clazz = jniEnv->FindClass("com/nhn/gametalk/api/internal/GTInternalAndroidUtil");
//
//		if(clazz == NULL)
//			return NULL;
//
//		clazz = (jclass)jniEnv->NewGlobalRef((jobject)clazz);
//	}
//
//	return clazz;
//}


/* implementation of the _splitpath runtime library function with ANSI character strings */
static void _splitpath(const CHAR* path, CHAR* drv, CHAR* dir, CHAR* name, CHAR* ext) {
	const CHAR* end; /* end of processed string */
	const CHAR* p; /* search pointer */
	const CHAR* s; /* copy pointer */

	/* extract drive name */
	if (path[0] && path[1] == ':') {
		if (drv) {
			*drv++ = *path++;
			*drv++ = *path++;
			*drv = '\0';
		}
	} else if (drv)
		*drv = '\0';

	/* search for end of string or stream separator */
	for (end = path; *end && *end != ':';)
		end++;
	/* search for begin of file extension */
	for (p = end; p > path && *--p != '\\' && *p != '/';)
		if (*p == '.') {
			end = p;
			break;
		}

	if (ext)
		for (s = end; (*ext = *s++);)
			ext++;

	/* search for end of directory name */
	for (p = end; p > path;)
		if (*--p == '\\' || *p == '/') {
			p++;
			break;
		}

	if (name) {
		for (s = p; s < end;)
			*name++ = *s++;

		*name = '\0';
	}

	if (dir) {
		for (s = path; s < p;)
			*dir++ = *s++;

		*dir = '\0';
	}
}


size_t GTCommonAPI::StrLen(LPCWSTR str)
{
#if defined(USE_SDK_UTF8)
	return str ? strlen(str) : 0;
#else
	return str ? wcslen(str) : 0;
#endif
}	

LPCWSTR GTCommonAPI::GetUUID(void)
{
	// Usage : _GenerateP2PID
	if(strlen(g_szUUID) <= 0)
	{
		GetDeviceUniqueId(g_szUUID, sizeof(g_szUUID));
	}

	return g_szUUID;
}


void GTCommonAPI::GetDeviceUniqueId(char *result, size_t size)
{
	// Usage : Make_AESCryptKey_For_RSACache (16 Length)
	JNIEnv *env = jniGetJNIEnvOnCurrentThreads();

	static jclass clazz;
	static jmethodID mid;
	if(NULL == clazz)
	{
		clazz = env->FindClass("com/nhn/gametalk/api/internal/GTInternalDeviceID");
		clazz = (jclass)env->NewGlobalRef(clazz);
		mid = env->GetStaticMethodID(clazz, "getDeviceId", "()Ljava/lang/String;");
	}

	jstring jstr = (jstring)env->CallStaticObjectMethod(clazz, mid);
	string cstr = CBindUtil::getStringFromJString(env, jstr);
	snprintf(result, size-1, "%s", cstr.c_str());
}

void GTCommonAPI::GetDeviceModelInfo(char * manufacturer, size_t size_mf, char * model,  size_t size_md, char * product,  size_t size_mp)
{
	// Usage : Make_AESCryptKey_For_RSACache (16 Length)
	JNIEnv *env = jniGetJNIEnvOnCurrentThreads();

	static jclass clazz;
	static jmethodID mid_mf , mid_md, mid_mp;
	if(NULL == clazz)
	{
		clazz = env->FindClass("com/nhn/gametalk/api/internal/GTInternalDeviceID");
		clazz = (jclass)env->NewGlobalRef(clazz);
		mid_mf = env->GetStaticMethodID(clazz, "getDeviceManufacturer", "()Ljava/lang/String;");
		mid_md = env->GetStaticMethodID(clazz, "getDeviceModel", "()Ljava/lang/String;");
		mid_mp = env->GetStaticMethodID(clazz, "getDeviceProduct", "()Ljava/lang/String;");
	}

	jstring jstr_mf = (jstring)env->CallStaticObjectMethod(clazz, mid_mf);
	jstring jstr_md = (jstring)env->CallStaticObjectMethod(clazz, mid_md);
	jstring jstr_mp = (jstring)env->CallStaticObjectMethod(clazz, mid_mp);
	string cstr_mf = CBindUtil::getStringFromJString(env, jstr_mf);
	string cstr_md = CBindUtil::getStringFromJString(env, jstr_md);
	string cstr_mp = CBindUtil::getStringFromJString(env, jstr_mp);
	snprintf(manufacturer, size_mf-1, "%s", cstr_mf.c_str());
	snprintf(model, size_md-1, "%s", cstr_md.c_str());
	snprintf(product, size_mp-1, "%s", cstr_mp.c_str());
}

void GTCommonAPI::TimeToSystemTime(__time32_t t, LPSYSTEMTIME pst)
{
	struct tm *ptm;
	ptm = _localtime32(&t);
	
	if(!pst)
		return;
	
	pst->wYear = ptm->tm_year + 1900;
	pst->wMonth = ptm->tm_mon + 1;
	pst->wDay = ptm->tm_mday;
	pst->wHour = ptm->tm_hour;
	pst->wMinute = ptm->tm_min;
	pst->wSecond = ptm->tm_sec;
	pst->wMilliseconds = 0;
	pst->wDayOfWeek = ptm->tm_wday;
}


__time32_t GTCommonAPI::SystemTimeToTime(LPSYSTEMTIME pst)
{
	struct tm *ptm;

	__time32_t t = time(NULL);
	ptm = _localtime32(&t);
	
	if(!pst)
		return 0;
	
	ptm->tm_year = pst->wYear - 1900;
	ptm->tm_mon = pst->wMonth - 1;
	ptm->tm_mday = pst->wDay;
	ptm->tm_hour = pst->wHour;
	ptm->tm_min = pst->wMinute;
	ptm->tm_sec = pst->wSecond;
	
	return mktime(ptm);
}


unsigned long long GTCommonAPI::GetFileSize(const char *filename)
{
	struct stat filestatus;
	int ret = stat(filename, &filestatus );
	if(ret != 0)
		return 0;
	return filestatus.st_size;
}


void GTCommonAPI::WriteTextFile(const char *filename, const char *data)
{
	// THERE IS NO USE IN ALL PROJECT..
	ASSERT(false);
}



size_t GTCommonAPI::WriteDataFile(const char *filename, const void *data, size_t size)
{
	COMMONAPI_LOG("WriteDataFile : %s, size = %d", filename, size);

	char szPath[MAX_PATH] = {0, };
	strcpy(szPath, filename);

	// get path from full-path
	_splitpath(filename, NULL, szPath, NULL, NULL);

	CreateDirectory(szPath);
		
	FILE *fp = fopen(filename, "w+");
	if(NULL == fp)
	{
		COMMONAPI_LOG("ERROR WriteDataFile : %s, size = %d", filename, size);
		return 0;
	}
	
	size_t ret = fwrite(data, 1, size, fp);
	fclose(fp);

	return ret;
}


bool GTCommonAPI::DeleteFile(const char *src)
{	
	CBindJavaFile file(src);
	file.makeObject();
	return file.deleteFile();
//
//    if(strlen(src) <= 0)
//        return false;
//
//	if(IsDirectory(src))
//		return false;
//
//	JNIEnv *env = NULL;
//	jclass clazz = __getJNIFileManager(&env);
//	if(NULL == clazz)
//	{
//		COMMONAPI_LOG("GTCommonAPI::DeleteFile : FAIL TO GET FILEMANATER JAVA CLASS");
//		return false;
//	}
//
//	static jmethodID mid;
//	if(mid == NULL)
//		mid = env->GetStaticMethodID(clazz, "deleteFile", "(Ljava/lang/String;)Z");
//	jstring str = env->NewStringUTF(src);
//	bool ret = env->CallStaticBooleanMethod(clazz, mid, str);
//	env->DeleteLocalRef(str);
//	return ret;
}


bool GTCommonAPI::MoveFile(const char *src, const char *dst)
{
	CBindJavaFile file(src);
	file.makeObject();
	bool ret = file.moveFile(dst);
	return ret;
//
//	JNIEnv *env = NULL;
//	jclass clazz = __getJNIFileManager(&env);
//	if(NULL == clazz)
//	{
//		COMMONAPI_LOG("GTCommonAPI::MoveFile : FAIL TO GET FILEMANATER JAVA CLASS");
//		return false;
//	}
//
//	static jmethodID mid;
//	if(mid == NULL)
//		mid  = env->GetStaticMethodID(clazz, "moveFile", "(Ljava/lang/String;Ljava/lang/String;)Z");
//	jstring str1 = env->NewStringUTF(src);
//	jstring str2 = env->NewStringUTF(dst);
//	bool ret = env->CallStaticBooleanMethod(clazz, mid, str1, str2);
//	env->DeleteLocalRef(str1);
//	env->DeleteLocalRef(str2);
//	return ret;
}


bool GTCommonAPI::CopyFile(const char *src, const char *dst)
{
//	JNIEnv *env = NULL;
//	jclass clazz = __getJNIFileManager(&env);
//	if(NULL == clazz)
//	{
//		COMMONAPI_LOG("GTCommonAPI::CopyFile : FAIL TO GET FILEMANATER JAVA CLASS");
//		return false;
//	}
//
//	static jmethodID mid;
//	if(mid == NULL)
//		mid = env->GetStaticMethodID(clazz, "copyFile", "(Ljava/lang/String;Ljava/lang/String;)Z");
//	jstring str1 = env->NewStringUTF(src);
//	jstring str2 = env->NewStringUTF(dst);
//	bool ret = env->CallStaticBooleanMethod(clazz, mid, str1, str2);
//	env->DeleteLocalRef(str1);
//	env->DeleteLocalRef(str2);
//
	return false;
}


bool GTCommonAPI::CreateDirectory(const char *path)
{
	CBindJavaFile file(path);
	file.makeObject();

	bool ret = file.createDirectory();

	COMMONAPI_LOG("ERROR CreateDirectory : %s, ret = %d", path, ret);

	return ret;
//
//	JNIEnv *env = NULL;
//	jclass clazz = __getJNIFileManager(&env);
//	if(NULL == clazz)
//	{
//		COMMONAPI_LOG("GTCommonAPI::CreateDirectory : FAIL TO GET FILEMANATER JAVA CLASS");
//		return false;
//	}
//
//	static jmethodID mid;
//	if(mid == NULL)
//		mid = env->GetStaticMethodID(clazz, "createDirectory", "(Ljava/lang/String;)Z");
//	jstring str = env->NewStringUTF(path);
//	bool ret = env->CallStaticBooleanMethod(clazz, mid, str);
//	env->DeleteLocalRef(str);
//	return ret;
}


char * GTCommonAPI::ReadDataFileAlloc(const char *filename, size_t &resultsize)
{
	resultsize = GetFileSize(filename);
	if(resultsize <= 0)
		return NULL;
	
	char *resultdata = new char[resultsize + 1];
	memset(resultdata, 0, resultsize + 1);
	
	resultsize = ReadDataFile(filename, resultdata, resultsize);
	
	if(resultsize <= 0)
	{
		delete [] resultdata;
		return NULL;
	}
	
	return resultdata;
}


size_t GTCommonAPI::ReadDataFile(const char *filename, char *result, size_t size)
{
	COMMONAPI_LOG("ReadDataFile : %s : %d", filename, size);
	int fd = open(filename, O_RDONLY);
	if(fd <= 0)
	{
		return 0;
	}
	
	char buf[2048];
	DWORD dwTotalRead = 0;
	DWORD dwRead;
	DWORD dwFileSize = GTCommonAPI::GetFileSize(filename);
	
	while(dwTotalRead < dwFileSize)
	{
		dwRead = read(fd, buf, sizeof(buf));
		
		ASSERT(dwFileSize >= dwTotalRead + dwRead);
		
		memcpy(result + dwTotalRead, buf, dwRead);
		dwTotalRead += dwRead;
	}
	close(fd);
	
	return dwTotalRead;
}

void GTCommonAPI::GetLocalAppPath(char *result, size_t size)
{
	if(NULL == gBindJavaAndroidContext)
	{
		memset(result, 0x0, size);	// NULL STRING..
		return;
	}

	string str = gBindJavaAndroidContext->getExternalCacheDir();
	snprintf(result, size-1, "%s", str.c_str());
}


bool GTCommonAPI::IsDirectory(const char *path)
{
	struct stat filestatus;
	int ret = stat(path, &filestatus );
	if(ret != 0)
		return false;

	return S_ISDIR(filestatus.st_mode);
}


bool GTCommonAPI::IsFileExist(const char *fileFullPath)
{
    struct stat stFileInfo; 
    bool blnReturn; 
    int intStat; 
    
    // Attempt to get the file attributes 
    if(stat(fileFullPath, &stFileInfo) == 0)
        return true;
    
    return false;
}

std::string GTCommonAPI::GetOsVersion()
{
	JNIEnv *env = jniGetJNIEnvOnCurrentThreads();

	jclass clazz = env->FindClass("android/os/Build$VERSION");
	jfieldID fid = env->GetStaticFieldID(clazz, "SDK_INT", "I");
	int nSDKNum = env->GetStaticIntField(clazz, fid);

	char szVersion[20] = {0, };
	sprintf(szVersion, "%d", szVersion);

	return string(szVersion);
}

std::string GTCommonAPI::GetDeviceVersion()
{
	// NOTHING TO USE...
	ASSERT(false);
	return "arm";
}


void GTCommonAPI::PrintLog(const char *log)
{
	LOGE2(__FUNCTION__, log);
}


int GTCommonAPI::GTSetting::ReadIntSettingBundle(const char *strKeyName)
{
	// KEH_ANDROID_BUNDLE_SETTING
	if(strcmp(strKeyName, "noise_suppress") == 0)
	{
		return -15;
	}
	if(strcmp(strKeyName, "noise_canceler") == 0)
	{
		return 1;
	}
	if(strcmp(strKeyName, "mainThreadMode") == 0)
	{
		return 1;
	}
	if(strcmp(strKeyName, "tcpNoDelay") == 0)
	{
		return 0;
	}
	if(strcmp(strKeyName, "silence_optimize") == 0)
	{
		return 10;
	}
	if(strcmp(strKeyName, "speex_quality") == 0)
	{
		return 6;
	}
	if(strcmp(strKeyName, "speex_complexity") == 0)
	{
		return 3;
	}

	return 0;
}

std::string GTCommonAPI::GTSetting::ReadStringSettingBundle(const char *strKeyName)
{
	// KEH_ANDROID_BUNDLE_SETTING
	if(strcmp(strKeyName, "codec") == 0)
	{
		return "speex";
	}

	// KEH_ANDROID_THREAD
	if(strcmp(strKeyName, "thread_priority") == 0)
	{
		//return "balanced";
		return "max_audio";
	}
	return "";
}

void GTCommonAPI::GTSetting::SetFloatValue(const char *strKeyName, const float value)
{

}

bool GTCommonAPI::GTSetting::GetFloatValue(const char *strKeyName, float *out)
{
	return false;
}

// EOF
